import com.dashrava.utils.Random;
import openfl.display.BitmapData;
import openfl.display.Shape;
class TerrainMap {
// Make the map Into an island.  0.39 makes almost every map Into an
// island;0.1 makes almost no map Into an island.
public var ISLAND_EFFECT:Float=0.01;

public var SIZE:Int;
public var SEED:Int;

public var altitude:Array<Array<Int>>;
public var moisture:Array<Array<Int>>;

public function new(size:Int, seed:Int){
	SIZE=size;
	SEED=seed;
	altitude=make2dArray(SIZE, SIZE);
	moisture = make2dArray(SIZE, SIZE);
}

public function generate():Void {

	// Generate 3-channel perlin noise and copy 2 of the channels out
	var b:BitmapData = new BitmapData(SIZE, SIZE);

	//b.perlinNoise(SIZE, SIZE, 4, SEED, false, false);

	var perlin:OptimizedPerlin = new OptimizedPerlin(SEED);
	perlin.fill(b, 0, 0, 0);

	var s:Shape=new Shape();

	// NOTE:if we remembered the equalization and other parameters,
	// we could zoom in at least on the altitude map by using the
	// perlinNoise()size and offsets parameters to regenerate
	// portions of the map at higher resolution. This could be useful
	// when drawing the detail map.
	//equalizeTerrain(b);

	// Overlay this on an "island" multiplier map
	// Based on http://www.ridgenet.net/~jslayton/FunWithWilburVol6/index.html
	for(x in 0 ... SIZE){
		for(y in 0 ... SIZE){
			var radiusSquared:Float=(x - SIZE * 0.5)*(x - SIZE * 0.5)+(y - SIZE * 0.5)*(y - SIZE * 0.5);
			radiusSquared +=Math.pow(Math.max(Math.abs(x - SIZE * 0.5), Math.abs(y - SIZE * 0.5)), 2);
			var max_radiusSquared:Float=SIZE * SIZE / 4;
			radiusSquared /=max_radiusSquared;
			radiusSquared +=Random.value()* 0.1;
			radiusSquared *=ISLAND_EFFECT;
			var island_multiplier:Float=Math.exp(-radiusSquared / 4)- radiusSquared;
			island_multiplier=island_multiplier * island_multiplier * island_multiplier;

			var c:UInt =b.getPixel(x, y);
			var height:Int=(c>>8)& 0xff;
			height=Std.int(island_multiplier * height);
			if(height<0)
				height=0;
			if(height>255)
				height=255;
			c=(c & 0xffff00ff)|(height<<8);
			b.setPixel(x, y, c);
		}
	}

	//equalizeTerrain(b);

	// Extract information from bitmap
	for(x in 0...SIZE){
		for(y in 0...SIZE){
			var c:Int=b.getPixel(x, y);
			altitude[x][y]=(c>>8)& 0xff;
			moisture[x][y]=c & 0xff;
		}
	}

}

/*public function equalizeTerrain(bitmap:BitmapData):Void {
	// Adjust altitude histogram so that it's roughly quadratic and
	// water histogram so that it's roughly linear
	var histograms:Array<Array<Float>>=bitmap.histogram(bitmap.rect);
	var G:Array<Float>=histograms[1];
	var B:Array<Float>=histograms[2];
	var g:Int=0;
	var b:Int=0;
	var green:Array<Dynamic>=new Array(256);
	var blue:Array<Dynamic>=new Array(256);
	var cumsumG:Float=0.0;
	var cumsumB:Float=0.0;
	for(i in 0...256){
		cumsumG +=G[i];
		cumsumB +=B[i];
		green[i]=(g * g / 255)<<8;// Int to green color value
		blue[i]=(b * b / 255);// Int to blue color value
		while(cumsumG>SIZE * SIZE * Math.sqrt(g / 256.0)&& g<255){
			g++;
		}
		while(cumsumB>SIZE * SIZE *(b / 256.0)&& b<255){
			b++;
		}
	}
	bitmap.paletteMap(bitmap, bitmap.rect, new Point(0, 0), null, green, blue, null);

}*/

public function make2dArray(w:Int, h:Int):Array<Array<Int>>{
	var v:Array<Array<Int>> = new Array<Array<Int>>();
	for(x in 0...w) {
		v[x]=new Array<Int>();
		for(y in 0...h){
			v[x][y]=0;
		}
	}
	return v;
}
}