package com.dashrava.utils;

/**
 * ...
 * @author Orava
 */
class GraphicUtils {

	/**
	 * Scale all components of a hex.
	 * @param	hex
	 * @param	scale
	 * @return	Scaled hex
	 */

	public static function scalehex(hex:Int, scale:Float):Int {

		var r:Int=(hex>>16	)& 255;
		var g:Int=(hex>>8	)& 255;
		var b:Int=(hex		)& 255;

		r=clampInt(r * scale);
		g=clampInt(g * scale);
		b=clampInt(b * scale);

		var col:Int=r<<16 | g<<8 | b;

		return col;

	}

	public static function clampInt(val:Float, min:Int=0, max:Int=255):Int {

		if(val<min)return min;
		if(val>max)return max;
		return Std.int(val);
	}

	/**
	 * Get brightness/luminance of hex
	 * @param	hex
	 * @return
	 */
	public static function hexBrightness(hex:Int):Float {
		var rgb:Array<Dynamic>=hexToRGB(hex);

		var R:Int=rgb[0];
		var G:Int=rgb[1];
		var B:Int=rgb[2];

		return 0.241 *(R * R)+ 0.691 *(G * G)+ 0.068 *(B * B);
	}

	/**
	 * Converts hex to RGB
	 * @param	hex
	 * @return Array [r,g,b]
	 */

	public static function hexToRGB(hex:Int):Array<Dynamic> {
		var rgb:Array<Dynamic>=new Array();
		rgb[0]=hex>>16;
		rgb[1]=hex>>8 & 0xFF;
		rgb[2]=hex & 0xFF;

		if(rgb[0]<0){ // Fix 32-bit
			rgb[0]=rgb[0] + 256;
		}

		return rgb;
	}

	/**
	 * Converts RGB to hex
	 * @param	red
	 * @param	green
	 * @param	blue
	 * @return hex Int
	 */
	public static function RGBToHex(R:Int, G:Int, B:Int):Int {

		return (R & 0xFF) << 16 | (G & 0xFF) << 8 | (B & 0xFF);
	}

}